###################################################
### chunk number 1: 
###################################################
## set options
options(prompt = "R> ", scipen=3, digits=4)
## for sampling
set.seed(1234)


###################################################
### chunk number 2: 
###################################################
library("seriation")

data("iris")
x <- as.matrix(iris[-5])
x <- x[sample(seq_len(nrow(x))),]
d <- dist(x)


###################################################
### chunk number 3: 
###################################################
## simple seriation
o <- seriate(d)
o


###################################################
### chunk number 4: 
###################################################
head(get_order(o), 15)


###################################################
### chunk number 5: 
###################################################
## plot seriation result
pimage(d, main = "Random")
pimage(d, o, main = "Reordered")

###################################################
### chunk number 6: 
###################################################

###################################################
### chunk number 7: 
###################################################
## compare criteria
cbind(random = criterion(d), reordered = criterion(d, o))

###################################################
### chunk number 8: 
###################################################

###################################################
### chunk number 9: 
###################################################
## Matrix shading of the iris data matrix
pimage(x, main = "Random", axes = 0)
text(1:dim(x)[2], par("usr")[3] - (par("usr")[4] - par("usr")[3]) / 20,
    srt = 45, adj = 1, labels =labels(x)[[2]], xpd = TRUE)
o_2mode <- c(o, ser_permutation(seq_len(ncol(x))))
pimage(x, o_2mode, main = "Reordered", axes = 0)
text(1:dim(x)[2], par("usr")[3] - (par("usr")[4] - par("usr")[3]) / 20,
    srt = 45, adj = 1, labels =labels(permute(x, o_2mode))[[2]], xpd = TRUE)
## without rotated labels:
## pimage(x, main = "Random")
## o_2mode <- c(o, ser_permutation(seq_len(ncol(x))))
## pimage(x, o_2mode, main = "Reordered")




###################################################
### chunk number 10: 
###################################################
## Comparing different seriation methods
methods <- c("TSP","Chen", "ARSA", "HC", "GW", "OLO")
o <- sapply(methods, FUN = function(m) seriate(d, m), simplify = FALSE)


###################################################
### chunk number 11: 
###################################################
timing <- sapply(methods, FUN = function(m) system.time(seriate(d, m)), 
    simplify = FALSE)


###################################################
### chunk number 12: 
###################################################
## do matrix shading for different seriation methods
tmp <- lapply(o, FUN = function(x) pimage(d, x, main = get_method(x[[1]])))


###################################################
### chunk number 13: 
###################################################


###################################################
### chunk number 14:
###################################################

###################################################
### chunk number 15: 
###################################################
## Dendrograms for the seriation with HC and GW
def.par <- par(no.readonly = TRUE)
layout(t(1:2))
plot(o[["HC"]][[1]], labels = FALSE, main = "Dendrogram HC")
symbols(74.7,.5, rect = matrix(c(4, 3), ncol=2), add= TRUE, 
    inches = FALSE, lwd =2)

plot(o[["GW"]][[1]], labels = FALSE, main = "Dendrogram GW")
symbols(98.7,.5, rect = matrix(c(4, 3), ncol=2), add= TRUE, 
    inches = FALSE, lwd =2)
par(def.par)


###################################################
### chunk number 16: 
###################################################
## compare the values of the loss/merit functions for the different 
## seriation methods
crit <- sapply(o, FUN = function(x) criterion(d, x))
t(crit)


###################################################
### chunk number 17: 
###################################################
## plot some results
def.par <- par(no.readonly = TRUE)
m <- c("Path_length", "AR_events", "Moore_stress")
layout(matrix(seq_along(m), ncol=1))
tmp <- lapply(m, FUN = function(i) dotchart(crit[i,], sub = i))
par(def.par)


###################################################
### chunk number 18: 
###################################################
## use registry
show_seriation_methods("dist")
show_seriation_methods("matrix")


###################################################
### chunk number 19: 
###################################################
list_seriation_methods("matrix")


###################################################
### chunk number 20: 
###################################################
## add new method
seriation_method_identity <- function(x, control) {
           lapply(dim(x), seq)
}


###################################################
### chunk number 21: 
###################################################
## use new method
set_seriation_method("matrix", "identity", seriation_method_identity,
    "Identity order")

set_seriation_method("array", "identity", seriation_method_identity,
    "Identity order")


###################################################
### chunk number 22: 
###################################################
show_seriation_methods("matrix")

o <- seriate(matrix(1, ncol=3, nrow=4), "identity")
o

get_order(o, 1)
get_order(o, 2)


###################################################
### chunk number 23: 
###################################################
x <- scale(x, center = FALSE)


###################################################
### chunk number 24: 
###################################################
hmap(x)


###################################################
### chunk number 25:
###################################################
hmap(x, hclustfun = NULL)

###################################################
### chunk number 26: 
###################################################

###################################################
### chunk number 27: 
###################################################


###################################################
### chunk number 28: 
###################################################
data("Irish")
orig_matrix <- apply(Irish[,-6], 2, rank)


###################################################
### chunk number 29: 
###################################################
o <- c(
    seriate(dist(orig_matrix, "minkowski", p = 1), method ="TSP"),
    seriate(dist(t(orig_matrix), "minkowski", p = 1), method = "TSP")
)
o


###################################################
### chunk number 30: 
###################################################

###################################################
### chunk number 31:
###################################################
## bertin plot
bertinplot(orig_matrix)


###################################################
### chunk number 32: bertin2
###################################################
bertinplot(orig_matrix, o)


###################################################
### chunk number 33: binary1
###################################################
data("Townships")

bertinplot(Townships, options = list(panel=panel.squares, spacing = 0, 
    frame = TRUE))


###################################################
### chunk number 34: 
###################################################
## to get consistent results
set.seed(5)


###################################################
### chunk number 35: binary2
###################################################
o <- seriate(Townships, method = "BEA", control = list(rep = 10))
bertinplot(Townships, o, options = list(panel=panel.squares, spacing = 0,
    frame = TRUE))


###################################################
### chunk number 36: 
###################################################
rbind(original = criterion(Townships), reordered = criterion(Townships, o))


###################################################
### chunk number 37: 
###################################################
data("iris")
iris <- iris[sample(seq_len(nrow(iris))),]
d <- dist(as.matrix(iris[-5]), method = "euclidean")


###################################################
### chunk number 38: dissplot1 eval=FALSE
###################################################
## plot original matrix
dissplot(d, method = NA)


###################################################
### chunk number 39: dissplot2 eval=FALSE
###################################################
## plot reordered matrix
dissplot(d, options = list(main = "Dissimilarity plot with seriation"))


###################################################
### chunk number 40: 
###################################################

###################################################
### chunk number 41: 
###################################################
set.seed(1234)


###################################################
### chunk number 42: 
###################################################
l <- kmeans(d, 10)$cluster


###################################################
### chunk number 43: 
###################################################
 res <- dissplot(d, labels = l,
     options = list(main = "Dissimilarity plot - standard"))


###################################################
### chunk number 44: 
###################################################

###################################################
### chunk number 45: 
###################################################
res


###################################################
### chunk number 46: 
###################################################
 plot(res, options = list(main = "Seriation - threshold",
     threshold = 1.5))


###################################################
### chunk number 47: 
###################################################
table(iris[res$order, 5], res$label)[,res$cluster_order]


###################################################
### chunk number 48: 
###################################################
data("ruspini")
d <- dist(ruspini)
l <- kmeans(d, 3)$cluster
## visualize the clustering
dissplot(d, labels = l)

